/**
 * \example terminal
 */

#include <hardware/btn-hardware.h>
#include <dev/usartavr.h>
#include <terminal/nut-cmds.h>
#include <terminal/btn-terminal.h>
#include <led/btn-led.h>
#include <sys/timer.h>

#include "program_version.h"

/**
 * Global Variables
 */


/**
 * init_terminal function that initializes the hardware, led, and terminal
 * Use tab-tab to see the registered commands once the program is running.
 */

void init_terminal(void)
{
    // serial baud rate
    u_long baud = 57600;
    u_long cpu_crystal;
    u_long nut_tick_freq;

    // init app uart
    NutRegisterDevice(&APP_UART, 0, 0);
    freopen(APP_UART.dev_name, "r+", stdout);
    _ioctl(_fileno(stdout), UART_SETSPEED, &baud);

    // hello world!
    printf("\n# ------------------------------------------------------\n");
    printf("# Welcome to BTnut (c) 2006 ETH Zurich\n");
    printf("# program version: %s\n", PROGRAM_VERSION);
    cpu_crystal = NutGetCpuClock();
    nut_tick_freq = NutGetTickClock();
    printf("# running @ %u.%04u MHz, NutFreq=%ul Hz\n",
           (int) (cpu_crystal / 1000000UL), (int) ((cpu_crystal - (cpu_crystal / 1000000UL) * 1000000UL) / 100),
        (int) nut_tick_freq);
    printf("# -----------------------------------------------------\n");

    // terminal init
    btn_terminal_init(stdout, "[btnut]$ ");
    nut_cmds_register_cmds();

    // give hint
    printf("Terminal initialized (hit tab twice for a list of commands)\n\r");
}

/**
 * main function. should never return.
 */

int main(void)
{
    // hardware init
    btn_hardware_init();
    btn_led_init(1);
    
    // terminal init
    init_terminal();
       
    // terminal mode
    btn_terminal_run(BTN_TERMINAL_NOFORK, 0);

    for (;;) { NutSleep(5000); }
}




