/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */
 
/**
 * @file bt_hci_event.h
 *
 * \author Mathias Payer <payerm@student.ethz.ch>
 * 
 * \date 2004/04/18 14:11:09 
 * 
 * @brief Handles hci events.
 *
 */
 
#include <bt/bt_hci_defs.h>
#include <bt/bt_defs.h>

// Events from host controller to host

#define BT_HCI_INQUIRY_COMPLETE				     0x01
#define BT_HCI_INQUIRY_RESULT				     0x02
#define BT_HCI_CONNECTION_COMPLETE			     0x03
#define BT_HCI_CONNECTION_REQUEST			     0x04
#define BT_HCI_DISCONNECTION_COMPLETE			 0x05
#define BT_HCI_REMOTE_NAME_REQUEST_COMPLETE		 0x07
#define BT_HCI_COMMAND_COMPLETE				     0x0E
#define BT_HCI_COMMAND_STATUS				     0x0F
#define BT_HCI_ROLE_CHANGE				         0x12
#define BT_HCI_NUMBER_OF_COMPLETED_PACKETS		 0x13
#define BT_HCI_MODE_CHANGE_EVENT                 0x14
#define BT_HCI_PIN_CODE_REQUEST                  0x16
#define BT_HCI_LINK_KEY_REQUEST                  0x17  
#define BT_HCI_LINK_KEY_NOTIFICATION             0x18
#define BT_HCI_MAX_SLOTS_CHANGED			     0x1B
#define CONNECTION_PACKET_TYPE_CHANGED           0x1D
#define BT_HCI_INQUIRY_RESULT_WITH_RSSI			 0x22
#define BT_HCI_VENDOR_SPECIFIC				     0xFF	//<< Vendor specific events for ericsson rok 101007

// Not (yet) implemented:
#define AUTHENTICATION_COMPLETE                  0x06   // not impl
#define ENCRIPTION_CHANGE                        0x08
#define CHANGE_CONNECTION_LINK_KEY_COMPLETE      0x09   // not impl
#define MASTER_LINK_KEY_COMPLETE                 0x0a   // not impl
#define READ_REMOTE_SUPPORTED_FEATURES_COMPLETE  0x0b
#define READ_REMOTE_VERSION_INFORMATION_COMPLETE 0x0c
#define QOS_SETUP_COMPLETE                       0x0d
#define HARDWARE_ERROR                           0x10
#define FLUSH_OCCURED                            0x11
#define RETURN_LINK_KEYS                         0x15   // not impl
#define DATA_BUFFER_OVERFLOW                     0x1a



/*******************************************************************************
 * _bt_hci_handle_event
 ******************************************************************************/
/**
 * @brief Handles (or dispatches) all incoming events
 *
 * Handles an event and updates the stack
 * Some events are handled externally (see below)
 *
 * Short events (like inquiry_result) are handled directly
 * inside this function
 *
 * @param stack The bt-stack to use
 * @param pkt the event packet
 *
 */
void _bt_hci_handle_event(struct btstack *stack, struct bt_hci_pkt_evt *pkt);

/*******************************************************************************
 * _bt_hci_handle_event_cmd_complete
 ******************************************************************************/
/**
 * @brief Handles BT_HCI_COMMAND_COMPLETE events
 *
 * Handles cmd_complete events and adjusts the stack
 *
 * @param stack The bt-stack to use
 * @param pkt the event packet
 *
 */
void _bt_hci_handle_event_cmd_complete(struct btstack *stack, struct bt_hci_pkt_evt *pkt);


/*******************************************************************************
 * _bt_hci_handle_event_cmd_status
 ******************************************************************************/
/**
 * @brief Handles BT_HCI_COMMAND_STATUS events
 *
 * Handles cmd_complete events and adjusts the stack
 *
 * @param stack The bt-stack to use
 * @param pkt the event packet
 *
 */
void _bt_hci_handle_event_cmd_status(struct btstack *stack, struct bt_hci_pkt_evt *pkt);

