/*
 * Copyright (C) 2000-2006 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 *
 */

/* dev/stream_buffer.h - buffer for stream data used in the RFCOMM
 *                       device driver
 *
 * 2005.12.15 Mustafa Yuecel <yuecelm@ee.ethz.ch>
 *
 */

#ifndef _DEV_STREAM_BUFFER_H_
#define _DEV_STREAM_BUFFER_H_

#include <sys/types.h>
#include <sys/mutex.h>

typedef struct stream_buffer {
    u_char * payload;   // pointer to the ring buffer
    u_short head;   // head position in array
    u_short tail;   // tail position in array (means next will be written there)
    u_short total_payload;   // summarized payload length of each buffer entry (offset excluded)
    u_short max_payload;   // maximum of allowed total_payload
    MUTEX * mutex;   // mutex for safe buffer usage
} stream_buffer_t;

stream_buffer_t * stream_buffer_init(u_short max_payload);
u_short stream_buffer_put(stream_buffer_t * sb, u_char * buffer, u_short len);
u_short stream_buffer_get(stream_buffer_t * sb, u_char * buffer, u_short len);
u_short stream_buffer_size(stream_buffer_t * sb);
void stream_buffer_destroy(stream_buffer_t * sb);

#endif
